package luokat;
import java.io.*;
import java.util.Random;

/** Pelaaja-luokka. Pitää kirjaa pelaajien id:stä, 
 * @author Sakari
 * @version 1.3.2025
 *
 */
public class Pelaaja {
    
    
    private String nimi;
    private int id;
    private static int seuraavaId = 1;
    
    
    

//    /** Pelaaja-luokan muodostaja
//     * @param nimi Pelaajan nimi
//     */
//    public Pelaaja(String nimi) {
//        this.nimi = nimi;
//    }
//    
    
    /** Metodi, joka hakee pelaajan nimen
     * @return Pelaajan nimen
     * @example
     * <pre name="test">
     *   Pelaaja pasi = new Pelaaja();
     *   pasi.vastaaPasi();
     *   pasi.getNimi() =R= "pasi .*";
     * </pre>
     */
    public String getNimi() {
        return this.nimi;
    }
    
    
    /** Antaa seuraavalle pelaajalle oikean id:n
     * @return uuden pelaajan id:n
     * @example
     * <pre name="test">
     * Pelaaja makipeto = new Pelaaja();
     * Pelaaja makimahoton = new Pelaaja();
     * Pelaaja vyonsolki = new Pelaaja();
     * makipeto.getId() === 0;
     * makipeto.rekisteroi();
     * makimahoton.rekisteroi();
     * vyonsolki.rekisteroi();
     * makipeto.getId() === 1;
     * makimahoton.getId() === 2;
     * vyonsolki.getId() === 3;
     * </pre>
     */
    public int rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
        return id;
    }
    
    /** Metodi, joka hakee pelaajan id:n
     * @return pelaajan id:n
     */
    public int getId() {
        return this.id;
    }
    
    
    /** Apumetodi, jolla sijoitetaan "pasi + arvottu nimi" pelaajalle.
     * @param arvottuNimi pelaajalle asetettava arvottu nimi
     */
    public void vastaaPasi(String arvottuNimi) {
        this.nimi=arvottuNimi;
    }
    
    /** 
     * Ylläolevan apumetodin apumetodi, joka arpoo nimen.
     */
    public void vastaaPasi() {
        String uusiNimi = arvoNimi();
        vastaaPasi(uusiNimi);
    }
    
    /** Apualiohjelma joka arpoo pelaajalle 
     * 6-kirjaimisen nimen.
     * @return arvottu 6 kirjaiminen nimi.
     * @example
     * <pre name="test">
     * Pelaaja pasi = new Pelaaja();
     * pasi.arvoNimi() =R= "pasi .*";
     * </pre>
     */
    public String arvoNimi() {
        Random r = new Random();
        StringBuilder uusiNimi = new StringBuilder();
        uusiNimi.append("pasi ");
        String aakkoset = "abcdefghijklmnopqrstuvxyz";
        for (int i = 0; i < 6; i++) {
            uusiNimi.append(aakkoset.charAt(r.nextInt(aakkoset.length())));
        }
        return uusiNimi.toString();
    }
    
    /**
     * Tulostetaan henkilön tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", this.id, 3) + "  " + this.nimi); 
    }
    
    /**
     * Tulostetaan henkilön tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    

    
    
    /** Testipääohjelma Pelaaja-luokalle. 
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Pelaaja pasi = new Pelaaja();
        System.out.println(pasi.getNimi());
        pasi.rekisteroi();
        System.out.println(pasi.getId());
        pasi.vastaaPasi();
        System.out.println(pasi.getNimi());
        pasi.tulosta(System.out);
    }

}
