package luokat;



/** Pelaajat-luokka. Pitää huolta pelaajalistasta.
 * Osaa lisätä ja poistaa jäsenen 
 * @author Sakari
 * @version 1.3.2025
 *
 */
public class Pelaajat {
    
    private static final int MAX_JASENIA   = 4;
    private int lkm =0;
    private String tiedostonNimi = "";
    private Pelaaja [] pelaajaLista = new Pelaaja[MAX_JASENIA];
    
    

    /**
     * Parametriton muodostaja Pelaajat 
     * (Oletusmuodostaja)
     */
    public Pelaajat() {
        
    }
    
    /**
     * Palauttaa Pelaajien lukumäärän
     * @return Pelaajien lukumäärä
     */
    public int getLkm() {
        return lkm;
    }

    /**
     * Lisää uuden Pelaaja-olion tietorakenteeseen.  
     * @param uusi lisättävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi
     * @throws SailoException jos pelaajalista on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja();
     * pelurit.getLkm() === 0;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 1;
     * pelurit.lisaaPelaaja(pasi2); pelurit.getLkm() === 2;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 3;
     * pelurit.annaPelaaja(0) === pasi1;
     * pelurit.annaPelaaja(1) === pasi2;
     * pelurit.annaPelaaja(2) === pasi1;
     * pelurit.annaPelaaja(1) == pasi1 === false;
     * pelurit.annaPelaaja(1) == pasi2 === true;
     * pelurit.annaPelaaja(3) === pasi1; #THROWS IndexOutOfBoundsException 
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 4;
     * pelurit.lisaaPelaaja(pasi1);  #THROWS SailoException
     * </pre>
     */
    public void lisaaPelaaja(Pelaaja uusi) throws SailoException{
        if (this.lkm >= pelaajaLista.length) throw new SailoException("Liikaa pelaajia!!");
        pelaajaLista[lkm] = uusi;
        lkm++;
    }
    
    /** Aliohjelma joka poistaa pelaajan pelaajalistasta, 
     * mikäli pelaaja ylipäätään on pelaajalistassa.
     * @param poistettava pelaaja joka yritetään poistaa
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja(), pasi3 = new Pelaaja();
     * pasi1.rekisteroi();
     * pasi2.rekisteroi();
     * pasi3.rekisteroi();
     * pelurit.lisaaPelaaja(pasi1);
     * pelurit.lisaaPelaaja(pasi2);
     * pelurit.lisaaPelaaja(pasi3);
     * pelurit.getLkm() === 3;
     * pelurit.poistaPelaaja(pasi2);
     * pelurit.getLkm() === 2;
     * pelurit.poistaPelaaja(pasi3);
     * pelurit.getLkm() === 1;
     * pelurit.poistaPelaaja(pasi1);
     * pelurit.getLkm() === 0;
     * </pre>
     */
    public void poistaPelaaja(Pelaaja poistettava) {
        for (int i = 0; i<pelaajaLista.length; i++) {
            if (pelaajaLista[i].getId() == poistettava.getId()) 
            {
                Pelaaja[] tilapaisLista = new Pelaaja[pelaajaLista.length];
                for (int j = 0; j<i; j++) {                         //kopioidaan kohtaa i aiemmat pelaajat tilapäislistaan.
                    tilapaisLista[j] = pelaajaLista[j];
                }
                for (int k = i; k<tilapaisLista.length-1; k++) {    //kopioidaan i:n jälkeiset pelaajat tilapäislistaan
                    if (pelaajaLista[k+1] != null) {                //tarkistetaan onko null
                        tilapaisLista[k] = pelaajaLista[k+1];       //jos ei null, kopioidaan tilapäiseen
                    }
                    else break;                                     //kun löytyy null (pelaajien välissä ei voi olla null)
                }
                pelaajaLista = tilapaisLista;                       
                lkm--;                                              
                return;
            }
        }
        return; //TODO TÄHÄN JOKIN VAROITUS; 
    }
    
    /** Palauttaa i:nnen jäsenen viitteen. 
     * @param i int-indeksi jonka kohdalla oleva Pelaaja
     * halutaan.
     * @return etsityn Pelaaja-olion.
     * @throws IndexOutOfBoundsException jos annettu indeksi i
     * on laiton. 
     */
    public Pelaaja annaPelaaja(int i) throws IndexOutOfBoundsException{
        if (i<0 || i>=lkm) throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return pelaajaLista[i];
    }
    
    
    /**
     * Lukee pelaajat tiedostosta. (Kesken)
     * @param hakemisto missä pelaajat.dat sijaitsee
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedostonNimi = hakemisto + "/pelaajat.dat";
        throw new SailoException("Ei osata vielä lukea tiedostoa " + tiedostonNimi);
    }

    
    /**
     * Tallentaa pelaajiston tiedostoon. (Kesken)
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        throw new SailoException("Ei osata vielä tallettaa tiedostoa " + tiedostonNimi);
    }

    
    /**
     * Metodi, joka tulostaa kaikki pelaajalistassa olevat 
     * pelaajat
     */
    public void tulostaPelaajat() {
        System.out.println("Pelaajat:");
        for (Pelaaja peluri : pelaajaLista) {
            if (peluri != null) {
                System.out.println(peluri.getNimi());
            }
            
        }
    }
    
    /** Testipääohjelma Pelaajat-luokalle
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        //TODO
        Pelaaja pasi = new Pelaaja();
        pasi.vastaaPasi();
        Pelaaja pasi2 = new Pelaaja();
        pasi2.vastaaPasi();
        Pelaajat pelurit = new Pelaajat();
        pasi.rekisteroi();
        pasi2.rekisteroi();
        try {
            pelurit.lisaaPelaaja(pasi);
            pelurit.lisaaPelaaja(pasi2);
            pelurit.tulostaPelaajat();
        }
        catch (SailoException ex) {
            System.out.print(ex.getMessage());
        }
        pelurit.poistaPelaaja(pasi);
        pelurit.tulostaPelaajat();
        
    }
    
    
}
