package luokat;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Random;

/** Rata 
 * @author Sakari
 * @version 10.3.2025
 *
 */
public class Rata {
    private String nimi = "";
    private int id;
    private int parasTulos; 
    private int parhaanId;
    private static int seuraavaId = 1;
    
    
    
    /**
     * Radan muodostaja
     */
    public Rata() {
        //ei tarvita vielä
    }
    
    /**
     * @param parhaanId kuka heittänyt 
     * radalla parhaan tuloksen.
     * @example
     * <pre name="test">
     * Rata laajis = new Rata(1);
     * laajis.getParhaanId() === 1;
     * </pre>
     */
    public Rata(int parhaanId) {
        this.parhaanId = parhaanId;
    }
    
    /**
     * @return parhaan tuloksen heittäneen pelaajan id
     */
    public int getParhaanId() {
        return this.parhaanId;
    }
    
      
    /**
     * @return radan parhaan tuloksen
     */
    public int getParasTulos() {
        return this.parasTulos;
    }
    /** Metodi, joka hakee pelaajan nimen
     * @return Pelaajan nimen
     * @example
     * <pre name="test">
     *   Rata laajis = new Rata();
     *   laajis.vastaaLaajis(1);
     *   laajis.getNimi() =R= "laajis .*";
     * </pre>
     */
    public String getNimi() {
        return this.nimi;
    }
    
    
    /** Antaa seuraavalle pelaajalle oikean id:n
     * @return uuden pelaajan id:n
     * @example
     * <pre name="test">
     * Rata makipeto = new Rata();
     * Rata makimahoton = new Rata();
     * Rata vyonsolki = new Rata();
     * makipeto.getId() === 0;
     * makipeto.rekisteroi();
     * makimahoton.rekisteroi();
     * vyonsolki.rekisteroi();
     * makipeto.getId() === 1;
     * makimahoton.getId() === 2;
     * vyonsolki.getId() === 3;
     * </pre>
     */
    public int rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
        return id;
    }
    
    /** Metodi, joka hakee radan id:n
     * @return pelaajan id:n
     */
    public int getId() {
        return this.id;
    }
    
    
    /** Apumetodi, jolla sijoitetaan "laajis + arvottu nimi" radalle.
     * @param arvottuNimi radalle asetettava arvottu nimi
     */
    public void vastaaLaajis(String arvottuNimi) {
        this.nimi=arvottuNimi;
        this.parasTulos = arvottuNumero(); 
    }
    
    /** 
     * Ylläolevan apumetodin apumetodi, joka arpoo nimen.
     * @param i kenen parhaaksi tulokseksi laitetaan
     */
    public void vastaaLaajis(int i) {
        this.parhaanId = i;
        String uusiNimi = arvoNimi();
        vastaaLaajis(uusiNimi);
    }
    
    /** Apualiohjelma joka arpoo radalle 
     * 3-kirjaimisen nimen.
     * @return laajis + arvottu 3 kirjaiminen nimi.
     * @example
     * <pre name="test">
     * Rata laajis = new Rata();
     * laajis.arvoNimi() =R= "laajis .*";
     * </pre>
     */
    public String arvoNimi() {
        Random r = new Random();
        StringBuilder uusiNimi = new StringBuilder();
        uusiNimi.append("laajis ");
        String aakkoset = "abcdefghijklmnopqrstuvxyz";
        for (int i = 0; i < 3; i++) {
            uusiNimi.append(aakkoset.charAt(r.nextInt(aakkoset.length())));
        }
        return uusiNimi.toString();
    }
    
    /**
     * @return arvotun numeron
     */
    public int arvottuNumero() {
        int palautus = random(-10, 10);
        return palautus;
    }
    
    /**
     * @param ala alaraja
     * @param yla ylaraja
     * @return random kokonaisluvun valitulta väliltä
     */
    public int random(int ala, int yla) {
        double rand = (yla - ala)*Math.random() +ala;
        return (int)Math.round(rand);
    }
    
    /**
     * Tulostetaan radan tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", this.id) + "  " + this.nimi); 
    }
    
    /**
     * Tulostetaan radan tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    

    
    
    /** Testipääohjelma Rata-luokalle
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Rata laajis = new Rata();
        laajis.vastaaLaajis(1);
        laajis.tulosta(System.out);
        laajis.rekisteroi();
        int p = laajis.getParhaanId();
        System.out.println(p);
        laajis.tulosta(System.out);
    }

}
